//
//  CalculatorBrain.swift
//  Calculator
//
//  Created by Sameh Fakhouri on 12/5/16.
//  Copyright © 2016 Sameh Fakhouri. All rights reserved.
//

import Foundation



class CalculatorBrain {
    
    
    private var accumulator = 0.0
    
    func setOperand(operand: Double) {
        accumulator = operand
    }
    
    private var operations: Dictionary<String, Operation> = [
        "π" : Operation.Constant(M_PI),
        "e" : Operation.Constant(M_E),
        "c" : Operation.Constant(0.0),
        "√" : Operation.UnaryOperation(sqrt),
        "×" : Operation.BinaryOperation({$0 * $1}),
        "÷" : Operation.BinaryOperation({$0 / $1}),
        "−" : Operation.BinaryOperation({$0 - $1}),
        "+" : Operation.BinaryOperation({$0 + $1}),
        "=" : Operation.Equals
    ]
    
    
    private enum Operation {
        case Constant(Double)
        case UnaryOperation((Double) -> Double)
        case BinaryOperation((Double, Double) -> Double)
        case Equals
    }
    
    
    func performOperation(symbol: String) {
        if let operation = operations[symbol] {
            switch operation {
            case .Constant(let myValue) :
                accumulator = myValue
                
            case .UnaryOperation(let function):
                accumulator = function(accumulator)
                
            case .BinaryOperation(let function):
                executePendingOperation()
                pending = PendingBinaryOperationInfo(binaryFunction: function,
                                                     firstOperand: accumulator)
            case .Equals:
                executePendingOperation()
            }
        }
        
    }
    
    
    private func executePendingOperation() {
        if (pending != nil) {
            accumulator = pending!.binaryFunction(pending!.firstOperand, accumulator)
            pending = nil
        }
    }
    
 
    private var pending: PendingBinaryOperationInfo?
    
    private struct PendingBinaryOperationInfo {
        var binaryFunction: (Double, Double) -> Double
        var firstOperand: Double
    }

    
    var result: Double {
        get {
            return accumulator
        }
    }
    
    
    
    
    
    
    
    
    
}
